@@ -1,4 +1,6 @@
-import logging, mpd
+import logging
+import mpd
+
 logging.basicConfig(level=logging.DEBUG)
 client = mpd.MPDClient()
 client.connect("localhost", 6600)
